==> YounifyConnectSDK-Android-Sample-0.9.15.tgz: app/src/main/java/tv/younify/sdk/connectapp/YounifyRootView.kt <==
package tv.younify.sdk.connectapp

import android.app.AlertDialog
import android.content.Context
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.SearchView
import androidx.activity.OnBackPressedCallback
import androidx.appcompat.app.AppCompatActivity
import androidx.core.view.MenuHost
import androidx.core.view.MenuProvider
import androidx.fragment.app.Fragment
import androidx.fragment.app.activityViewModels
import androidx.fragment.app.commit
import androidx.fragment.app.replace
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import kotlinx.coroutines.Job
import kotlinx.coroutines.launch
import tv.younify.sdk.connect.FetchContentCategoryResult
import tv.younify.sdk.connect.StreamingCategory
import tv.younify.sdk.connect.StreamingContent
import tv.younify.sdk.connect.StreamingContentType
import tv.younify.sdk.connect.StreamingService
import tv.younify.sdk.connectapp.databinding.FragmentYounifyRootViewBinding
import kotlin.coroutines.cancellation.CancellationException

class YounifyRootView : Fragment(), MenuProvider, ImageRequestListener {
    private var services: List<StreamingService>? = null
    private lateinit var binding: FragmentYounifyRootViewBinding
    private lateinit var adapter: YounifyCategoriesAdapter
    private var loadJob: Job? = null
    private val servicesViewModel: YounifyServicesViewModel by activityViewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                services = it.getParcelableArrayList("services",StreamingService::class.java)
            } else {
                @Suppress("DEPRECATION")
                services = it.getParcelableArrayList("services")
            }
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        binding = FragmentYounifyRootViewBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val menuHost: MenuHost = requireActivity()
        menuHost.addMenuProvider(this, viewLifecycleOwner, Lifecycle.State.RESUMED)

        (activity as AppCompatActivity).supportActionBar?.title = "Categories"

        binding.activityIndicator.visibility = View.VISIBLE
        binding.categoriesRecyclerView.layoutManager =
            GridLayoutManager(this.context, 1, GridLayoutManager.VERTICAL, false)

        adapter = YounifyCategoriesAdapter(lifecycleScope, YounifyCategoryItemsAdapter.OnTapListener { content: StreamingContent, service: StreamingService ->
            (activity as AppCompatActivity).supportActionBar?.title = content.title
            val bundle = Bundle()
            bundle.putParcelable("content", content)
            bundle.putParcelable("contentService", service)
            if (content.type == StreamingContentType.Folder) {
                val servicesList = services
                bundle.putParcelableArrayList(
                    "services",
                    if (servicesList != null) ArrayList(servicesList) else ArrayList()
                )
                bundle.putString("searchTerm", "")
                bundle.putBoolean("isSearch", false)

                this.activity?.supportFragmentManager?.commit {
                    setCustomAnimations(R.anim.slide_in, 0)
                    replace<YounifyBrowseFragment>(
                        R.id.fragment_container_younify_browse_view,
                        args = bundle
                    )
                    setReorderingAllowed(true)
                    addToBackStack(null)
                }
            } else {
                this.activity?.supportFragmentManager?.commit {
                    setCustomAnimations(R.anim.slide_in, 0)
                    replace<YounifyDetailsFragment>(
                        R.id.fragment_container_younify_browse_view,
                        args = bundle
                    )
                    setReorderingAllowed(true)
                    addToBackStack(null)
                }
            }
        }, this)

        binding.categoriesRecyclerView.adapter = adapter
    }

    override suspend fun fetchImage(url:String) : ByteArray? {
        return servicesViewModel.fetchImage(url)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        val callback = object : OnBackPressedCallback(true) {
            override fun handleOnBackPressed() {
               val count = activity?.supportFragmentManager?.backStackEntryCount!!
                if (count != 1) {
                    activity?.supportFragmentManager?.popBackStack()
                    if (count == 2)
                        (activity as AppCompatActivity).supportActionBar?.title = "Categories"

                    return
                }

                loadJob?.cancel()
                (activity as AppCompatActivity).supportActionBar?.title = "Services"
                activity?.supportFragmentManager?.popBackStack()
            }
        }

        requireActivity().onBackPressedDispatcher.addCallback(this, callback)
    }

    override fun onCreateMenu(menu: Menu, menuInflater: MenuInflater) {
        menuInflater.inflate(R.menu.younify_search_menu, menu)

        val searchViewItem = menu.findItem(R.id.action_younify_search)
        val searchView = searchViewItem?.actionView as SearchView

        searchView.setOnQueryTextListener(object : SearchView.OnQueryTextListener {
            override fun onQueryTextSubmit(query: String): Boolean {
                val servicesList = services
                val bundle = Bundle()
                bundle.putParcelable("content", null)
                bundle.putParcelableArrayList(
                    "services",
                    if (servicesList != null) ArrayList(servicesList) else ArrayList()
                )
                bundle.putString("searchTerm", query)
                bundle.putBoolean("isSearch", true)

                activity?.supportFragmentManager?.commit {
                    setCustomAnimations(R.anim.slide_in, 0)
                    replace<YounifyBrowseFragment>(
                        R.id.fragment_container_younify_browse_view,
                        args = bundle
                    )
                    setReorderingAllowed(true)
                    addToBackStack(null)
                }

                searchView.clearFocus()
                return true
            }

            override fun onQueryTextChange(newText: String): Boolean {
                return false
            }
        })
    }

    override fun onMenuItemSelected(menuItem: MenuItem): Boolean {
        return false
    }

    override fun onStart() {
        super.onStart()
        load()
    }

    private fun load() {
        loadJob = lifecycleScope.launch {
            try {
                val currentServices = services
                if (currentServices != null) {
                    val categories = servicesViewModel.fetchCategories()

                    adapter.startLoading()
                    binding.activityIndicator.visibility = View.VISIBLE
                    adapter.setCategories(categories)
                    binding.activityIndicator.visibility = View.GONE

                    servicesViewModel.fetchContent(
                        categories,
                        currentServices,
                        { service: StreamingService, error: Throwable? ->
                            if (error != null)
                                Log.e("YounifyRootView", "There was error fetching service ${service.name}")
                        },
                        { category: StreamingCategory, service: StreamingService, content: List<StreamingContent>, error: Throwable? ->
                            if (error != null)
                                Log.e("YounifyRootView", "There was error fetching category ${category.name} for service ${service.name}")

                            adapter.addCategoryContentItems(
                                    category,
                                    service,
                                    content)
                        },
                        { _: List<FetchContentCategoryResult>, _: Throwable? ->
                            Log.i("YounifyRootView", "Results for category when canceled or completed successfully")
                        }
                    )
                }
            }
            catch (ex: CancellationException) {
                throw ex
            }
            catch (cause: Throwable) {
                AlertDialog.Builder(context)
                    .setMessage("Error: ${cause.message}")
                    .setTitle("Error fetching content")
                    .setPositiveButton("OK") { _, _ -> }
                    .create()
                    .show()
            }
            finally {
                binding.activityIndicator.visibility = View.GONE
                adapter.finishLoading()
            }
        }
    }
}

==> YounifyConnectSDK-Android-Sample-0.9.17.tgz: app/src/main/java/tv/younify/sdk/connectapp/YounifyRootView.kt <==
package tv.younify.sdk.connectapp

import android.app.AlertDialog
import android.content.Context
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuInflater
import android.view.MenuItem
import android.view.View
import android.view.ViewGroup
import android.widget.SearchView
import androidx.activity.OnBackPressedCallback
import androidx.appcompat.app.AppCompatActivity
import androidx.core.view.MenuHost
import androidx.core.view.MenuProvider
import androidx.fragment.app.Fragment
import androidx.fragment.app.activityViewModels
import androidx.fragment.app.commit
import androidx.fragment.app.replace
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import kotlinx.coroutines.Job
import kotlinx.coroutines.launch
import tv.younify.sdk.connect.FetchContentCategoryResult
import tv.younify.sdk.connect.StreamingCategory
import tv.younify.sdk.connect.StreamingContent
import tv.younify.sdk.connect.StreamingContentType
import tv.younify.sdk.connect.StreamingService
import tv.younify.sdk.connectapp.databinding.FragmentYounifyRootViewBinding
import kotlin.coroutines.cancellation.CancellationException

class YounifyRootView : Fragment(), MenuProvider, ImageRequestListener {
    private var services: List<StreamingService>? = null
    private lateinit var binding: FragmentYounifyRootViewBinding
    private lateinit var adapter: YounifyCategoriesAdapter
    private var loadJob: Job? = null
    private val servicesViewModel: YounifyServicesViewModel by activityViewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                services = it.getParcelableArrayList("services",StreamingService::class.java)
            } else {
                @Suppress("DEPRECATION")
                services = it.getParcelableArrayList("services")
            }
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View {
        binding = FragmentYounifyRootViewBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val menuHost: MenuHost = requireActivity()
        menuHost.addMenuProvider(this, viewLifecycleOwner, Lifecycle.State.RESUMED)

        (activity as AppCompatActivity).supportActionBar?.title = "Categories"

        binding.activityIndicator.visibility = View.VISIBLE
        binding.categoriesRecyclerView.layoutManager =
            GridLayoutManager(this.context, 1, GridLayoutManager.VERTICAL, false)

        adapter = YounifyCategoriesAdapter(lifecycleScope, YounifyCategoryItemsAdapter.OnTapListener { content: StreamingContent, service: StreamingService ->
            (activity as AppCompatActivity).supportActionBar?.title = content.title
            val bundle = Bundle()
            bundle.putParcelable("content", content)
            bundle.putParcelable("contentService", service)
            if (content.type == StreamingContentType.Folder) {
                val servicesList = services
                bundle.putParcelableArrayList(
                    "services",
                    if (servicesList != null) ArrayList(servicesList) else ArrayList()
                )
                bundle.putString("searchTerm", "")
                bundle.putBoolean("isSearch", false)

                this.activity?.supportFragmentManager?.commit {
                    setCustomAnimations(R.anim.slide_in, 0)
                    replace<YounifyBrowseFragment>(
                        R.id.fragment_container_younify_browse_view,
                        args = bundle
                    )
                    setReorderingAllowed(true)
                    addToBackStack(null)
                }
            } else {
                this.activity?.supportFragmentManager?.commit {
                    setCustomAnimations(R.anim.slide_in, 0)
                    replace<YounifyDetailsFragment>(
                        R.id.fragment_container_younify_browse_view,
                        args = bundle
                    )
                    setReorderingAllowed(true)
                    addToBackStack(null)
                }
            }
        }, this)

        binding.categoriesRecyclerView.adapter = adapter
    }

    override suspend fun fetchImage(url:String) : ByteArray? {
        return servicesViewModel.fetchImage(url)
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        val callback = object : OnBackPressedCallback(true) {
            override fun handleOnBackPressed() {
               val count = activity?.supportFragmentManager?.backStackEntryCount!!
                if (count != 1) {
                    activity?.supportFragmentManager?.popBackStack()
                    if (count == 2)
                        (activity as AppCompatActivity).supportActionBar?.title = "Categories"

                    return
                }

                loadJob?.cancel()
                (activity as AppCompatActivity).supportActionBar?.title = "Services"
                activity?.supportFragmentManager?.popBackStack()
            }
        }

        requireActivity().onBackPressedDispatcher.addCallback(this, callback)
    }

    override fun onCreateMenu(menu: Menu, menuInflater: MenuInflater) {
        menuInflater.inflate(R.menu.younify_search_menu, menu)

        val searchViewItem = menu.findItem(R.id.action_younify_search)
        val searchView = searchViewItem?.actionView as SearchView

        searchView.setOnQueryTextListener(object : SearchView.OnQueryTextListener {
            override fun onQueryTextSubmit(query: String): Boolean {
                val servicesList = services
                val bundle = Bundle()
                bundle.putParcelable("content", null)
                bundle.putParcelableArrayList(
                    "services",
                    if (servicesList != null) ArrayList(servicesList) else ArrayList()
                )
                bundle.putString("searchTerm", query)
                bundle.putBoolean("isSearch", true)

                activity?.supportFragmentManager?.commit {
                    setCustomAnimations(R.anim.slide_in, 0)
                    replace<YounifyBrowseFragment>(
                        R.id.fragment_container_younify_browse_view,
                        args = bundle
                    )
                    setReorderingAllowed(true)
                    addToBackStack(null)
                }

                searchView.clearFocus()
                return true
            }

            override fun onQueryTextChange(newText: String): Boolean {
                return false
            }
        })
    }

    override fun onMenuItemSelected(menuItem: MenuItem): Boolean {
        return false
    }

    override fun onStart() {
        super.onStart()
        load()
    }

    private fun load() {
        loadJob = lifecycleScope.launch {
            try {
                val currentServices = services
                if (currentServices != null) {
                    val categories = servicesViewModel.fetchCategories()

                    adapter.startLoading()
                    binding.activityIndicator.visibility = View.VISIBLE
                    adapter.setCategories(categories)
                    binding.activityIndicator.visibility = View.GONE

                    servicesViewModel.fetchContent(
                        categories,
                        currentServices,
                        { service: StreamingService, error: Throwable? ->
                            if (error != null)
                                Log.e("YounifyRootView", "There was an error fetching service ${service.name}")
                        },
                        { category: StreamingCategory, service: StreamingService, content: List<StreamingContent>, error: Throwable? ->
                            if (error != null)
                                Log.e("YounifyRootView", "There was an error fetching category ${category.name} for service ${service.name}")

                            adapter.addCategoryContentItems(
                                    category,
                                    service,
                                    content)
                        },
                        { _: List<FetchContentCategoryResult>, _: Throwable? ->
                            Log.i("YounifyRootView", "Results for category when canceled or completed successfully")
                        }
                    )
                }
            }
            catch (ex: CancellationException) {
                throw ex
            }
            catch (cause: Throwable) {
                AlertDialog.Builder(context)
                    .setMessage("Error: ${cause.message}")
                    .setTitle("Error fetching content")
                    .setPositiveButton("OK") { _, _ -> }
                    .create()
                    .show()
            }
            finally {
                binding.activityIndicator.visibility = View.GONE
                adapter.finishLoading()
            }
        }
    }
}
